/*	Environment(machine)-specific basic functions follow 
	These are specified to access to MZ-80K (with 28 KB RAM) memory map.
	I/O ports are connected to PORTB of PIC32MX150F128B.
	To emulate the other computer, these must be modified.
*/

// _getCode() macro (as inline function) that fetches a code using PC, and increment PC.
#define _getCode() ((z80PC<0x1000)\
	?(\
		monitor[(z80PC++)]\
	):(\
		(z80PC<0x8000)\
		?(\
			RAM[(z80PC++)-0x1000]\
		):(\
			(z80PC<0xD000)\
			?(\
				0xC7\
			):(\
				(z80PC<0xE000)\
				?(\
					VRAM[(z80PC++)&0x03FF]\
				):(\
					0xC7\
				)\
			)\
		)\
	))
UINT8 getCode(){
	// Using getCode() as inline function for all will use much memory to exceed the maximum.
	return _getCode();
}

// readMemory() function uses 34 cycles max, 21 cycles typical
unsigned char readMemory(UINT16 addr){
	addr&=0xffff;
	if (addr<0x1000) return monitor[addr];
	if (addr<0x8000) return RAM[addr-0x1000];
	if (addr<0xD000) return 0xC7;
	if (addr<0xE000) return VRAM[addr & 0x03FF];
	if (addr<0xE004) {
		// 8255
		read825x[2]&=0x7f;
		read825x[2]|=g_Vsync?0x80:0; // Wait for V blank
		return read825x[addr & 0x000F];
	}
	if (addr<0xE008) {
		// 8253
		return read8253(addr);
	}
	if (addr<0xE009) {
		// Tempo
		return read825x[8];
	}
	return 0xC7; // RST00
}

// writeMemory() function takes 44 cycles max, 31 cycles typical.
void writeMemory(UINT16 addr, UINT8 data){
	addr&=0xffff;
	data&=0xff;
	if (addr<0x1000) return; // ROM region cannot be written.
	if (addr<0x8000) {
		RAM[addr-0x1000] = data;
		return;
	}
	if (addr<0xD000) return;
	if (addr<0xE000) {
		VRAM[addr & 0x03FF] = data;
		return;
	}
	if (addr<0xE004) {
		write8255(addr,data);
		return;
	}
	if (addr<0xE008) {
		write8253(addr,data);
		return;
	}
	if (addr<0xE009) {
		// on-off sound
		T3CONbits.ON=(data&0x01);
		return;
	}
}
// PORTB0-6 + PORTB12 are connected to I/O ports in Z80 emulation.
// readIO() function
unsigned char readIO(UINT8 addrL, UINT8 addrH){
	switch(addrL){
		case 0x80:
			return (TRISB&0x7f) | ((TRISB&0x1000)>>5);
		case 0x81:
			return (PORTB&0x7f) | ((PORTB&0x1000)>>5);
		case 0x82:
			return (ANSELB&0x7f) | ((ANSELB&0x1000)>>5);
		case 0x83:
			return (CNPUB&0x7f) | ((CNPUB&0x1000)>>5);
		case 0x84: // Read analog from RB0
		case 0x85: // Read analog from RB1
		case 0x86: // Read analog from RB2
		case 0x87: // Read analog from RB3
		default:
			return 0x00;
	}
}
// writeIO() function
void writeIO(UINT8 addrL, UINT8 addrH, UINT8 data){
	static unsigned short tstart;
	unsigned short tend;
	unsigned char result;
	switch(addrL){
		case 0x80:
			TRISB=TRISB&0xEF80 | data&0x7F | ((data&0x80)<<5);
			return;
		case 0x81:
			TRISB=TRISB&0xEF80 | data&0x7F | ((data&0x80)<<5);
			return;
		case 0x82:
			ANSELB=ANSELB&0xEF80 | data&0x7F | ((data&0x80)<<5);
			return;
		case 0x83:
			CNPUB=CNPUB&0xEF80 | data&0x7F | ((data&0x80)<<5);
			return;
		case 0x8F: // Casette tape recorder interface
			// Disable interrupt for video signal here, if needed (it will be enabled by sync-signal handling routine).
			switch (data) {
				case 0x00: // Read from casette tape and return
				default:
					// A/D converter reading routine will be here (result will be 0x20 or 0x00).
					read825x[2]&=0xdf;
					read825x[2]|=result;
					return;
				case 0x08: // Wait until signal will be H.
					read825x[2]&=0xdf;
					while(read825x[1]&0x08){// Exit loop if break key is pressed.
						// A/D converter reading routine will be here(read and wait for acquisition)
						if (0==result) continue;
						read825x[2]|=result;
						tstart=TMR1;
						return; 
					}
					return;
				case 0x26: // Wait for 350 usec after raising signal.
					// Disable some interrupts here.
					read825x[2]&=0xdf;
					tend=tstart+14000; // About 350 usec
					if (tstart<tend) {
						while(tstart<TMR1 && TMR1<tend);
					} else {
						while(tstart<TMR1);
						while(TMR1<tend);
					}
					// A/D converter reading routine will be here
					read825x[2]|=result;
					// Enable the disabled interrupts here.
					return;
			}
		default:
			return;
	}
}

/*	End of environment-specific basic functions */
